//  Copyright (c) 1999-2016  Pavel Rychly, Milos Jakubicek

tree grammar cqpTreeWalker;

options {
	language = C;
	tokenVocab = cqp;
	ASTLabelType = pANTLR3_BASE_TREE;
}

@includes {
	#include <finlib/fsop.hh>
	#include <finlib/regpref.hh>
	#include "frsop.hh"
	#include "corpus.hh"
	#include "parsop.hh"
	#include "cqpeval.hh"
	#include "config.hh"
#ifdef MANATEE_SKETCH_ENGINE
	#include "wmap.hh"
#endif
	#include <cstdlib>
	#include <iostream>
	#define STR(t) ((const char *) t->chars)
	#define INT(t) (atoi(STR(t)))
	#define LONG(t) (atol(STR(t)))
	#define LLONG(t) (atoll(STR(t)))
}

@header {
    // Copyright (c) 1999-2015  Pavel Rychly, Milos Jakubicek
}

@apifuncs {
    RECOGNIZER->displayRecognitionError = throwEvalQueryException;
}



prog [Corpus &c] returns [RangeStream *r = NULL]
@init{
	r = NULL;
}
	: query[c];

query [Corpus &c] returns [RangeStream *r, string desc]
	: within_containing[c] {$r=$within_containing.r;}
	| globcond[c] {$r=$globcond.r; $desc=$globcond.desc;}
	;

within_containing [Corpus &c] returns [RangeStream *r, string desc]
@init{
	bool not_within_containing = false;
	RangeStream *rs = NULL;
}
	: ^(KW_WITHIN (NOT {not_within_containing=true;})? p=query[c]
            (
				( s=structure[c] {rs=$s.r;} | q=query[c] {rs=$q.r;} )
			| 
				(NOT {not_within_containing=true;})? a=alignedpart[c, !not_within_containing] {rs=$a.r;}
			)
        )
	    {if (not_within_containing)
            $r = new RQnotInNode ($p.r, rs);
         else
            $r = new RQinNode ($p.r, rs);}
	| ^(KW_CONTAINING (NOT {not_within_containing=true;})? p=query[c]
            (
				( s=structure[c] {rs=$s.r;}| q=query[c] {rs=$q.r;})
			| 
				a=alignedpart[c, !not_within_containing] {rs=$a.r;}
			)
        )
	    {if (not_within_containing)
            $r = new RQnotContainNode ($p.r, rs);
         else
            $r = new RQcontainNode ($p.r, rs);}
	;

alignedpart [Corpus &c, bool add_labels] returns [RangeStream *r, string desc]
@init{
	Corpus *al_corp;
}    
	: 
	  ^(COLON al_name=ATTR {al_corp = c.get_aligned(STR($al_name.text)); al_corp->set_default_attr (c.get_default_attr()->name); }
        q=bigseq[*al_corp]
        {$r = c.map_aligned (al_corp, $q.r, add_labels);}
        )
    ;

globcond [Corpus &c] returns [RangeStream *r, string desc]
@init{
	FastStream *a, *b; RQFilterPos::Op posOp; RQFilterFreq::Op freqOp; int neg = 0;
}
    : 
        ^(BINAND bigseq[c] {$r=$bigseq.r;} 
            (^(DOT n1=NUMBER a1=ATTR 
                    (EQ  {posOp=RQFilterPos::F_EQ;}
                    |NOT EQ {posOp=RQFilterPos::F_NEQ;}
                    )
                    n2=NUMBER a2=ATTR)
                {$r= new RQFilterPos ($r, c.get_attr (STR($a1.text)),
                        c.get_attr (STR($a2.text)), posOp,
                        LONG($n1.text), LONG($n2.text));
                }
			|^(KW_FREQ n1=NUMBER a1=ATTR (NOT { neg=1; })?
					(EQ {freqOp=RQFilterFreq::F_EQ;}
					|LEQ {freqOp=RQFilterFreq::F_LEQ;}
					|GEQ {freqOp=RQFilterFreq::F_GEQ;}
					|LSTRUCT {freqOp=RQFilterFreq::F_GEQ; neg=1;}
					|RSTRUCT {freqOp=RQFilterFreq::F_LEQ; neg=1;}
					) f=NUMBER)
				{$r= new RQFilterFreq($r, c.get_attr (STR($a1.text)),
						freqOp, neg, LLONG ($n1.text), LLONG ($f.text));}
            )+
        )
    |
        bigseq[c] {$r = $bigseq.r;$desc = $bigseq.desc;}
    ;

structure [Corpus &c] returns [RangeStream *r, string desc]
@init{
	Structure *strc;
}	
	: 
	  sn=ATTR {strc = c.get_struct (STR($sn.text));}
	    (
			{$r = strc->rng->whole();}
	    | fs = onepos[*strc]
			{$r = strc->rng->part($fs.r);}
	    )
	;

bigseq [Corpus &c] returns [RangeStream *r, string desc]
@init{
	bool usesort=false;
	bool complement=false;
	RangeStream *rs = NULL;
	$r = NULL;
}	
	:
	  ^(SEQ (NOT {complement = true;})?
	  		(sp=seqpart[c] {$r = $sp.r; $desc=$sp.desc;})? 
			(
				(
	  			  fs=muquery[c] {rs = new Pos2Range ($fs.r, 0, 1);}
				| rep=repeat[c] {rs = $rep.r;}
				| wc=within_containing[c] {rs = $wc.r;}
				| b=bigseq[c] {rs = $b.r;}
				| ^(BEGSTRUCT s=structure[c])  
					{rs = new Pos2Range (new BegsOfRStream ($s.r), 0, 0);}
				| ^(ENDSTRUCT s=structure[c])  
					{rs = new Pos2Range (new EndsOfRStream ($s.r), 0, 0);}
				| ^(WHOLESTRUCT s=structure[c]) {rs = $s.r;}
				)
					{if ($r) {
                        $r = RQConcatNodeUnsort ($r, rs);
                        usesort = true;
                    } else {
                        $r = rs;
                    }}
				(p=seqpart[c]  
					{$r = RQConcatNodeUnsort ($r, $p.r); usesort=true; $desc=$p.desc;}
				)?
			)*
		)
        {if (usesort) $r = new RQSortBeg ($r);}
		{if (complement) $r = new RQoutsideNode ($r, c.size());}

	| ^(BINOR b1=bigseq[c] b2=bigseq[c]) {$r = new RQUnionNode ($b1.r,$b2.r); }
	;

seqpart [Corpus &c] returns [RangeStream *r, string desc]
@init{
	int count = 1;
	FastStream *fs = NULL;
}	
	: 
	  ^(LBRACKET res=labeledpos[c]) {fs = $res.ls;}
		(
			^(LBRACKET s=labeledpos[c]) 
			{fs = new QAndNode(fs, new QMoveNode ($s.ls, -count)); count++;}
		)*
		{$r = new Pos2Range (fs, 0, count); $desc=$res.desc;}
	;

labeledpos [Corpus &c] returns [FastStream *ls, string desc]
@init{
	int label=0;
}    
	: 
	  (
        ^(COLON n=NUMBER ^(LBRACKET s=onepos[c]))
            {label = LONG ($n.text);}
      | s = onepos[c]
      )
        {$ls = new AddLabel ($s.r, label); $desc=$s.desc;}

    ;

repeat [Corpus &c] returns [RangeStream *r, string desc]
@init{
	int rn1, rn2;
}	
	: 
	  ^(REPOPT 
			(
				p=query[c] ^(REPOPT n1=NUMBER n2=NUMBER)
				{
					rn1 = LONG ($n1.text);
					rn2 = LONG ($n2.text);
					if (rn1 == 0 && rn2 == 1) {
						$r = new RQOptionalNode ($p.r);
					} else {
						$r = new RQRepeatNode ($p.r, rn1, rn2);
					}
				}
			|
				^(LBRACKET fs=onepos[c]) ^(REPOPT n3=NUMBER n4=NUMBER)
				{
					rn1 = LONG ($n3.text);
					rn2 = LONG ($n4.text);
					if (rn1 == 0 && rn2 == 1) {
						$r = new RQOptionalNode (new Pos2Range ($fs.r));
					} else {
						$r = new RQRepeatFSNode ($fs.r, rn1, rn2);
					}
				}
			)
		)
	;


onepos [Corpus &c] returns [FastStream *r, string desc]
@init {
	int neg = 0, cmp = 0, thesnum = 0;
	string wm_base;
}
	: 
	  ^(BINAND a=onepos[c] b=onepos[c])   {$r = new QAndNode ($a.r,$b.r); $desc=$a.desc + " a " + $b.desc;}
	| ^(BINOR a=onepos[c] b=onepos[c])	{$r = new QOrNode ($a.r,$b.r); $desc=$a.desc + " nebo " + $b.desc;}
	| ^((EQ|LEQ { cmp = -1; } |GEQ { cmp = 1; }|EEQ { cmp = 2; }|TEQ { cmp = 3; })
		a1=ATTR n=NUMBER? (NOT { neg = 1; })? v1=REGEXP)
		{
			$desc="atribut ";
			$desc += STR($a1.text);
			if (neg)
				$desc += " nemá ";
			else
				$desc += " má ";
			$desc += "hodnotu '";
			$desc += STR($v1.text);
			$desc += "'";
			if (n)
				thesnum = INT ($n.text);
			if (!cmp) {
				$r = c.get_attr (STR($a1.text))->regexp2poss (STR($v1.text), false);
			} else if (cmp == 2) {
				PosAttr *pa = c.get_attr (STR($a1.text));
				$r = pa->id2poss (pa->str2id (STR($v1.text)));
#ifdef MANATEE_SKETCH_ENGINE
			} else if (cmp == 3) {
				$r = id2thesposs (&c, STR($a1.text), STR($v1.text), thesnum);
#endif
			} else {
				$r = c.get_attr (STR($a1.text))->compare2poss (STR($v1.text), cmp, false);
			}
			if (neg) {
				if ($r->final() == 0) {
					// empty stream
					delete $r;
					$r = new SequenceStream (0, c.size() -1, c.size());
				} else {
					$r = new QNotNode ($r, c.size());
				}
			}
		}
    | ^(NOT a=onepos[c])       {$r = new QNotNode ($a.r, c.size());}
	| ^(ANYPOS ())	{$r = new SequenceStream (0, c.size() -1, c.size());}
	| ^(n1=NUMBER n2=NNUMBER
		{
		Position beg = LLONG ($n1.text);
		Position end = llabs(LLONG($n2.text));
		$r = new SequenceStream (beg, end, c.size());
		})
	| ^(KW_SWAP n=NUMBER a=onepos[c])
		{
		Position coll = LONG($n.text);
		$r = new SwapKwicColl ($a.r, coll);
		}
	| ^(KW_CCOLL f=NUMBER t=NUMBER a=onepos[c])
		{
		int from = LONG($f.text);
		int to = LONG($t.text);
		$r = new ChangeLabel ($a.r, from, to);
		}
	| ^( (KW_WS { wm_base=c.get_conf("WSBASE");}
	     | KW_TERM { wm_base=c.get_conf("TERMBASE"); }
	     )
	    level=NUMBER seek=NUMBER)
		{
#ifdef MANATEE_SKETCH_ENGINE
		int l = INT($level.text);
		WMap *ws = new_WMap (wm_base, c.get_confpath(), l, STR($seek.text));
		std::vector<FastStream*> *svec = new std::vector<FastStream*>();
		svec->push_back (ws->poss());
		$r = new QOrVNode_clean_wmap (svec, ws);
#else
        throw EvalQueryException (": word sketches not supported");
#endif
		}
	| ^( (KW_WS { wm_base=c.get_conf("WSBASE"); }
	     | KW_TERM { wm_base=c.get_conf("TERMBASE"); }
	     )
	    w1=REGEXP gr=REGEXP w2=REGEXP)
		{
#ifdef MANATEE_SKETCH_ENGINE
		std::vector<FastStream*> *svec = new std::vector<FastStream*>();
		PosAttr *pa = c.get_attr (c.get_conf("WSATTR"));
		const char *locale = c.get_conf(c.get_conf("WSATTR") + ".LOCALE").c_str();
		WMap *ws0 = new_WMap (wm_base, c.get_confpath());
		const char *pat1 = STR($w1.text);
		const char *pat2 = STR($w2.text);
		regexp_pattern re1 (pat1, locale, c.get_conf("ENCODING").c_str());
		re1.compile();
		regexp_pattern re2 (pat2, locale, c.get_conf("ENCODING").c_str());
		re2.compile();
		do {
			if (re1.no_meta_chars() && !ws0->findid (pa->str2id (pat1)))
				break;
			else if (!re1.match (pa->id2str (ws0->getid())))
				continue;
			WMap *ws1 = ws0->nextlevel();
			Generator<int> *gen = ws1->regexp2ids (STR($gr.text));
			while (!gen->end()) {
				int id = gen->next();
				if (!ws1->findid(id))
					continue;
				WMap *ws2 = ws1->nextlevel();
				do {
					if (re2.no_meta_chars() && !ws2->findid (ws2->coll2id (pat2)))
						break;
					else if (!re2.match (ws2->id2coll (ws2->getid())))
						continue;
					svec->push_back (ws2->poss());
				} while (ws2->next() && !re2.no_meta_chars());
				delete ws2;
			}
			delete gen;
			delete ws1;
		} while (ws0->next() && !re1.no_meta_chars());
		if (svec->empty()) {
			delete ws0;
			delete svec;
			$r = new EmptyStream();
		} else
			$r = new QOrVNode_clean_wmap (svec, ws0);
#else
        throw EvalQueryException (": word sketches not supported");
#endif
		}
	;


muquery [Corpus &c] returns [FastStream *r, string desc]
	: 
	  ^(KW_UNION  a=mupart[c] b=mupart[c]) {$r = new QOrNode ($a.r,$b.r);}
	| ^(KW_MEET a=mupart[c] b=mupart[c]
			(n1=NUMBER n2=NUMBER
				{
					int i1 = LONG ($n1.text);
					int i2 = LONG ($n2.text);
					if (i1 == i2)
						$r = new QAndNode($a.r, new QMoveNode ($b.r, -i1));
					else
						$r = new BegsOfRStream (new RQinNode (new Pos2Range ($a.r),
									new Pos2Range ($b.r, -i2, -i1+1)));
				}
			|
				{$r = new QAndNode($a.r, new QMoveNode ($b.r, -1));}
			)
		)
	;

mupart [Corpus &c] returns [FastStream *r, string desc]
	:
	  muquery[c] { $r=$muquery.r; }
	| ^(LBRACKET labeledpos[c]) {$r=$labeledpos.ls;}
	;

// vim: syntax=antlr3
